import abc
import logging
from contextlib import contextmanager

import pydicom
from pynetdicom import AE
from pynetdicom.presentation import build_context

from pacs_config import PacsConfiguration

STATUS_SUCCESS = 0x0000


class PacsError(Exception):
    pass


class AbstractPacsClient(abc.ABC):

    @abc.abstractmethod
    def store_dicom_dataset(self, dataset: pydicom.Dataset):
        raise NotImplementedError


class PacsClient(AbstractPacsClient):

    def __init__(self, config: PacsConfiguration):
        self.ae_title = config.ae_title
        self.host = config.host
        self.port = config.port
        self.dimse_timeout = config.dimse_timeout

    @contextmanager
    def _association(self, dataset):
        self.ae = AE(self.ae_title)
        self.ae.requested_contexts = [build_context(dataset.SOPClassUID,
                                                    dataset.file_meta.TransferSyntaxUID)]
        assoc = self.ae.associate(self.host, self.port)
        assoc.dimse_timeout = self.dimse_timeout
        yield assoc
        assoc.release()

    def store_dicom_dataset(self, dataset: pydicom.Dataset):
        try:
            with self._association(dataset) as assoc:
                if assoc.is_established:
                    status = assoc.send_c_store(dataset)
                    if getattr(status, 'Status', None) == STATUS_SUCCESS:
                        logging.info("Saved dicom to Pacs")
                    else:
                        raise PacsError()
                else:
                    raise PacsError("Association not established")
        except Exception as e:
            logging.exception("Failed saving dicom to Pacs")
            if isinstance(e, PacsError):
                raise
            else:
                raise PacsError(repr(e))



import logging

import pika
from pika.exchange_type import ExchangeType

from aide.messaging.queue_config import QueueConfiguration


class Publisher():
    def __init__(self, config: QueueConfiguration):
        self.config = config

    def _connect(self):
        credentials = pika.PlainCredentials(username=self.config.username,
                                            password=self.config.password)

        connection_params = pika.ConnectionParameters(
            host=self.config.host,
            port=self.config.port,
            credentials=credentials)

        self._connection = pika.BlockingConnection(parameters=connection_params)
        logging.info('Connection opened')
        self._channel = self._connection.channel()
        # self._channel.add_on_close_callback(self._on_channel_closed)
        self._channel.exchange_declare(
            durable=True,
            exchange=self.config.model_exchange,
            exchange_type=ExchangeType.direct)
        self._channel.queue_declare(queue=self.config.queue, durable=True,
                                    arguments={"x-queue-type": "quorum"})
        self._channel.queue_bind(
            self.config.queue,
            self.config.model_exchange,
            routing_key=self.config.model_routing_key)

    def publish_message(self, message):
        # @todo this is not yet ready
        self._channel.basic_publish(message, None, None)



import os


class QueueConfiguration:
    def __init__(self, username="",
                 password="",
                 host="",
                 port="",
                 model_exchange="",
                 output_exchange="",
                 model_routing_key="",
                 output_routing_key="",
                 queue="",
                 prefetch_count=None,
                 connection_retry_limit=None,
                 connection_retry_wait=None):
        self.username = username if username != "" else os.getenv("RABBITMQ_USERNAME", "admin")
        self.password = password if password != "" else os.getenv("RABBITMQ_PASSWORD", "admin")
        self.host = host if host != "" else os.getenv("RABBITMQ_HOST", "localhost")
        self.port = port if port != "" else os.getenv("RABBITMQ_PORT", 5672)
        self.model_exchange = model_exchange if model_exchange != "" else os.getenv("RABBITMQ_MODEL_EXCHANGE", "model-exchange")
        self.output_exchange = output_exchange if output_exchange != "" else os.getenv("RABBITMQ_OUTPUT_EXCHANGE", "output-exchange")
        self.model_routing_key = model_routing_key if model_routing_key != "" else os.getenv("RABBITMQ_MODEL_ROUTING_KEY", "")
        self.output_routing_key = output_routing_key if output_routing_key != "" else os.getenv("RABBITMQ_OUTPUT_ROUTING_KEY", "input")
        self.prefetch_count = prefetch_count if prefetch_count is not None else int(os.getenv("RABBITMQ_PREFETCH_COUNT", 0))
        self.connection_retry_limit = connection_retry_limit if connection_retry_limit is not None else int(os.getenv("RABBITMQ_CONNECTION_RETRY_LIMIT", 5))
        self.connection_retry_wait = connection_retry_wait if connection_retry_wait is not None else int(os.getenv("RABBITMQ_CONNECTION_RETRY_WAIT", 30))
        self.queue = queue
